@@ -1,6 +1,6 @@
 /**
  *      @file    cipherSuite.c
- *      @version 692b1c6 (HEAD -> master, origin/master, origin/HEAD)
+ *      @version 950bba4 (HEAD -> master)
  *
  *      Wrappers for the various cipher suites..
  *      Enable specific suites at compile time in matrixsslConfig.h
@@ -2883,6 +2883,12 @@ const sslCipherSpec_t *sslGetCipherSpec(const ssl_t *ssl, uint16_t id)
 #ifdef VALIDATE_KEY_MATERIAL
         if (ssl->keys != NULL)
         {
+            if ((ssl->flags & SSL_FLAGS_SERVER) == 0)
+            {
+                /* Client: Just accept the cipher suite, because we do not
+                   know of server public key yet. */
+                return &supportedCiphers[i];
+            }
             if (haveKeyMaterial(ssl, supportedCiphers[i].type, 0)
                 == PS_SUCCESS)
             {
